pro imsum, nomIMAGES, nomMOZZ, methode_delta, interpole=interpole, mediane=mediane, moymed=moymed, $
		moyenne=moyenne, nonan=nonan, badpix=badpix, resolelement=resolelement, $
		fichier_clic=fichier_clic, listex=listex, listey=listey, $
		nouvellesbornes = bornes

;******************* methode de calcul des deltas ******************
case strcompress(strupcase(methode_delta),/remove_all) of
'LISTE': begin
	;L'utilisateur fournit une liste de la position de la meme etoile presente sur toutes
	;les images.

	;Construit les vecteurs de deltas x et y
	dimn = n_elements(nomIMAGES)
	deltax = fltarr(dimn)
	deltay = fltarr(dimn)
	for i=0,dimn-1 do begin
		deltax[i] = -(listex[i] - listex[0])
		deltay[i] = -(listey[i] - listey[0])
	endfor
end

'LISTE_RAFFINEE': begin
	;L'utilisateur fournit une liste de la position de la meme etoile presente sur toutes
	;les images, ce au pixel pres. Puis, imsum trouve les positions exactes des images les
	;unes par rapport aux autres, en extrayant la position des etoiles avec SExtractor.

	;Construit les vecteurs de deltas x et y
	dimn = n_elements(nomIMAGES)
	deltax = fltarr(dimn)
	deltay = fltarr(dimn)
	for i=0,dimn-1 do begin
		deltax[i] = -(listex[i] - listex[0])
		deltay[i] = -(listey[i] - listey[0])
	endfor
end

'1CLIC': begin
	;L'utilisateur clique sur une seule etoile commune a toutes
	;les images.
	if not keyword_set(fichier_clic) then begin
		;Determine le nom du fichier des clics par defaut
		tmp = strcompress(strsplit(nomMOZZ,'.fits',/regex,/extract)+'.clic',/remove_all)
		fichier_clic = tmp[0]
	endif
	if keyword_set(resolelement) eq 1 then begin
		refstar, nomIMAGES, listex, listey, resolelement=resolelement, $
			fichier_clic=fichier_clic, /absolu
	endif else begin
		refstar, nomIMAGES, listex, listey, resolelement=5.0, $
			fichier_clic=fichier_clic, /absolu
	endelse
	
	;Construit les vecteurs de deltas x et y
	dimn = n_elements(nomIMAGES)
	deltax = fltarr(dimn)
	deltay = fltarr(dimn)
	for i=0,dimn-1 do begin
		deltax[i] = -(listex[i] - listex[0])
		deltay[i] = -(listey[i] - listey[0])
	endfor

end

'AUTO': begin
	;La translation entre les images est trouvee automatiquement simplement en
	;correlant le profil horizontal et vertical des images. Rapide et efficace.
	;Il est preferable que le ciel soit soustrait des images et que les mauvais
	;pixels soient mis a NAN.
	imcorrel, nomIMAGES, deltax, deltay
end

'IMREF': begin
	;L'utilisateur clique sur 3 etoiles par images et sur une
	;image de reference.
end

'ENTETE': begin
	;Les deltas sont trouves automatiquement grace a SExtractor,
	;a la fonction srcor, et aux entetes fits qui donnent la position
	;du telescope. Suppose qu'il y a au moins quelques source
	;communes entre chaque image.
	print, 'Des deltas automatiques sont appliques aux images avant leur combinaison.'
	calculedxdyproche, nomIMAGES, deltax, deltay
end

'CONVOLUTION': begin
	;Les deltas sont trouves automatiquement grace, d'abord a la correlation croisee
	;entre les images, puis a SExtractor et a la fonction srcor pour peaufiner les deltas
	;Suppose qu'il y a au moins quelques sources communes entre chaque image.
	print, 'Des deltas automatiques sont appliques aux images avant leur combinaison.'
	calculedxdyproche, nomIMAGES, deltax, deltay, /convoluer
end

else: begin
	;Aucun delta n'est applique avant la combinaison des images.
	print, 'Aucun delta n''est applique aux images avant leur combinaison.'
	deltax = fltarr(n_elements(image))
	deltay = fltarr(n_elements(image))
end
endcase


;*******************************************************************
; DEBUT DE LA COMBINAISON DES IMAGES
;*******************************************************************

;*************************** keyword interpole *********************
if not keyword_set(interpole) then begin
	;On arrondit les deltas aux entiers les plus proches
	deltax = round(deltax)
	deltay = round(deltay)
	methode = strcompress(strupcase(methode_delta),/remove_all)
	if strcmp(methode,'LISTE') eq 1 or strcmp(methode,'1CLIC') eq 1 then begin
		listex = round(listex)
		listey = round(listey)
	endif
endif


;****************** Met les images dans un cube *******************

dimn = n_elements(nomIMAGES)

;Quelle est la taille de chaque image?
taillex = intarr(n_elements(nomIMAGES))
tailley = intarr(n_elements(nomIMAGES))
for i=0, n_elements(nomIMAGES)-1 do begin
	tmp = size(readfits(nomIMAGES[i],/silent))
	taillex[i] = tmp[1]
	tailley[i] = tmp[2]
endfor

minx = ceil(max(listex))
maxx = ceil(max(taillex-listex))
miny = ceil(max(listey))
maxy = ceil(max(tailley-listey))
dimx = minx+maxx
dimy = miny+maxy
bornex1 = minx-round(listex)
bornex2 = bornex1+taillex-1
borney1 = miny-round(listey)
borney2 = borney1+tailley-1

;Imprime le nom et les deltas pour chaque image
for i=0,dimn-1 do begin
	print, nomIMAGES[i],deltax[i],deltay[i]
endfor

;Fabrique une image de NAN
imagenan = fltarr(dimx,dimy) * !values.f_nan

;Lit les images fits dans un cube et applique l'interpolation si tel est le cas
if keyword_set(badpix) then begin
	imBADPIX = readfits(badpix,/silent)	;maintenant, badpix=NAN
endif
cube = fltarr(dimx,dimy,dimn)
for i=0,dimn-1 do begin
	;Met tous les pixels des images a NAN.
	print, 'Initialisation de l''image ',i
	cube[*,*,i] = imagenan
	print, 'Lecture de l''image ',i+1
	;Lit le temps de pose dans le header
	imagevec = readfits(nomIMAGES[i],entete,/silent)
	if keyword_set(badpix) then begin
		imagevec = imagevec * imBADPIX
	endif
	if keyword_set(interpole) then begin
	;NON-implemente encore
	;	interx = deltax[i] - round(deltax[i])
	;	intery = deltay[i] - round(deltay[i])
	;	grillex = findgen(dimx_uneimage) - interx
	;	grilley = findgen(dimy_uneimage) - intery
	;	cube[bornex1[i]:bornex2[i],borney1[i]:borney2[i],i] = interpolate(imagevec,grillex,grilley,/grid,cubic=-0.5)
	endif else begin
		cube[bornex1[i]:bornex2[i],borney1[i]:borney2[i],i] = imagevec
	endelse

endfor

;Si vous voulez voir le cube avant la combinaison...
writefits, 'zcube.fits',cube







goto, passe_version1

;*************** Fabrication du cube d'images **********************
;Trouve les dimensions de la mosaique finale et le point zero.
dimn = n_elements(nomIMAGES)
imtmp = readfits(nomIMAGES[0],/silent)
szima = size(imtmp)
dimx = szima[1] + (ceil(max(deltax)) - floor(min(deltax)))
dimy = szima[2] + (ceil(max(deltay)) - floor(min(deltay)))
x0 = round(min(deltax))
y0 = round(min(deltay))
print, x0, y0
;Conserve la dimension d'une image
dimx_uneimage = szima[1]
dimy_uneimage = szima[2]

;Imprime le nom et les deltas pour chaque image
for i=0,dimn-1 do begin
	print, nomIMAGES[i],deltax[i],deltay[i]
endfor

;Fabrique une image de NAN
imagenan = fltarr(dimx,dimy)
for i=0,dimx-1 do begin
	for j=0,dimy-1 do begin
		;imagenan[i,j] = 0.0
		imagenan[i,j] = !VALUES.F_NAN
	endfor
endfor

print, dimx_uneimage, dimy_uneimage


;Lit les images fits dans un cube et applique l'interpolation si tel est le cas
if keyword_set(badpix) then begin
	imBADPIX = readfits(badpix,/silent)	;maintenant, badpix=NAN
endif
cube = fltarr(dimx,dimy,dimn)
bornes = fltarr(dimn, 4)
for i=0,dimn-1 do begin
	;Met tous les pixels des images a NAN.
	print, 'Initialisation de l''image ',i
	cube[*,*,i] = imagenan
	;Trouve les bornes dans la mosaique.
	bornex1 = round(deltax[i]-x0)
	bornex2 = dimx_uneimage -1 + bornex1
	borney1 = round(deltay[i]-y0)
	borney2 = dimy_uneimage -1 + borney1
	bornes[i,*] = [bornex1,bornex2,borney1,borney2]
	print, bornex1,bornex2,borney1,borney2
	print, 'Lecture de l''image ',i+1
	;Lit le temps de pose dans le header
	imagevec = readfits(nomIMAGES[i],entete,/silent)
	if keyword_set(badpix) then begin
		imagevec = imagevec * imBADPIX
	endif
	if keyword_set(interpole) then begin
		interx = deltax[i] - round(deltax[i])
		intery = deltay[i] - round(deltay[i])
		grillex = findgen(dimx_uneimage) - interx
		grilley = findgen(dimy_uneimage) - intery
		cube[bornex1:bornex2,borney1:borney2,i] = interpolate(imagevec,grillex,grilley,/grid,cubic=-0.5)
	endif else begin
		cube[bornex1:bornex2,borney1:borney2,i] = imagevec
	endelse

endfor

;Si vous voulez voir le cube avant la combinaison...
writefits, 'zcube.fits',cube

passe_version1:


;************ combinaison des images: keywords mediane, moymed **************
print, 'Combine les images du cube...'
if keyword_set(mediane) then begin
	;On combine les images en faisant la mediane
	print, 'Combine les images par une mediane...'
	medarr, cube, mosaiquefinale
endif
if keyword_set(moymed) then begin
	;On combine en faisant une moyenne intelligente ou les pixels
	;deviants sont rejetes afin d'aller chercher le meilleur S/B
	;possible tout en rejetant les mauvais pixels comme la mediane.
	;Voir notes du 16 juillet 2003 pour comprendre.
	print, 'Combine les images par une moyenne intelligente qui rejete les mauvais pixels'
	print, 'et optimise le S/B de la mosaique resultante...'
	;TROUVE L'IMAGE MEDIANE
	print, '      Trouve la mediane...'
	medarr, cube, immediane

	;TROUVE L'IMAGE MOYENNE
	;-fabrique une image 'imflag' qui donne le nbr d'images sans nan pour chaque pixel
	;-fait la moyenne en faisant la somme des images divisee par imflag
	print, '      Trouve la moyenne...'
	cubeflag = intarr(dimx,dimy,dimn)
	cubeflag[*,*,*] = 1
	for z=0, dimn-1 do begin
		image = cube[*,*,z]
		imageflag = cubeflag[*,*,z]
		index = where(finite(image) ne 1,nindex)
		if nindex gt 0 then begin
			imageflag[index] = 0
		endif
		cubeflag[*,*,z] = imageflag
	endfor
	imflag = total(cubeflag,3)
	immoyenne = total(cube,3,/nan)/imflag

	;TROUVE LES PIXELS A INSPECTER ET FAIT LA MEDIANE POUR CES PIXELS AU LIEU DE LA MOYENNE
	print, '      Corrige les pixels deviants...'
	immoymed = immoyenne
	imdiff = abs(immoyenne - immediane)/immediane
	sigma_imdiff = abs(imdiff / ecart_median(imdiff))
	index = where(sigma_imdiff ge 5.0,nindex) ;plus grand que 5 sigmas d'ecart
	if nindex gt 0 then begin
		cubedeviant = fltarr(dimx,dimy,dimn)
		cubedeviant[*,*,*] = !VALUES.F_NAN
		for z = 0, dimn-1 do begin
			image = cube[*,*,z]
			imagedeviant = cubedeviant[*,*,z]
			imagedeviant[index] = image[index]
			cubedeviant[*,*,z] = imagedeviant
		endfor
		medarr, cubedeviant, imdeviant
		immoymed[index] = imdeviant[index]
	endif
	mosaiquefinale = immoymed
endif
if not keyword_set(mediane) and not keyword_set(moymed) then begin
	;(Par defaut) On combine en faisant la moyenne des images
	print, 'Combine les images par une moyenne...'
	;TROUVE L'IMAGE MOYENNE
	;-fabrique une image 'imflag' qui donne le nbr d'images sans nan pour chaque pixel
	;-fait la moyenne en faisant la somme des images divisee par imflag
	print, '      Trouve la moyenne...'
	cubeflag = intarr(dimx,dimy,dimn)
	cubeflag[*,*,*] = 1
	for z=0, dimn-1 do begin
		image = cube[*,*,z]
		imageflag = cubeflag[*,*,z]
		index = where(finite(image) ne 1,nindex)
		if nindex gt 0 then begin
			imageflag[index] = 0
		endif
		cubeflag[*,*,z] = imageflag
	endfor
	imflag = total(cubeflag,3)
	mosaiquefinale = total(cube,3,/nan)/imflag
endif


;*********************** keyword nonan *****************************
print, 'Enleve les nan de la mosaique...'
if keyword_set(nonan) then begin
	mosaiquefinale = effaceNAN(mosaiquefinale)
endif


;*************** Sauvegarde de la mosaique sur disque **************
print, 'Sauve la mosaique sur disque...'
writefits, nomMOZZ, mosaiquefinale

fin:
end


















function filtreOS, nomfichier

nomfichier=strcompress(nomfichier,/remove_all)

morceaux = strsplit(nomfichier, '/\',/extract)

if strcmp(!version.os_family, 'UNIX',/fold_case) eq 1 then begin
	path_final = strcompress('/' + strjoin( morceaux,'/', /single ))
endif else begin
	if strcmp(!version.os_family, 'Windows',/fold_case) eq 1 then begin
		path_final = strjoin( morceaux,'\', /single )
	endif
endelse

return,path_final

end














function effaceNAN, image, nanvaleur=nanvaleur

if not keyword_set(nanvaleur) then begin
	nanvaleur = median(image)
endif

szima = size(image)
dimx = szima[1]
dimy = szima[2]

for i=0, dimx-1 do begin
	for j=0, dimy-1 do begin
		if NOT FINITE(image[i,j]) then image[i,j] = nanvaleur
	endfor
endfor

return, image

end
