pro refstar, images, deltax, deltay, resolelement=resolelement, fichier_clic=fichier_clic, $
	absolu = absolu 


;refstar fait de la photometrie de profile d'une etoile sur les
;images reduites individuelles. Ensuite, refstar compare les mesures
;dans chacune des images pour voir si c'est constant et pour eliminer
;les mauvaises images.
;
;Parametres:
;resolelement : nombre d'elements de resolution pour une etoile, autrement dit:
;		fwhm (") divise par echelle du ccd ("/pixel)
;
;Revisions:
;2003/10/02 - On peut passer les positions absolues cliquees de chaque image plutot
;	      que les deltas par rapport a ces positions, pour ce faire, il faut mettre
;	      le keyword 'absolu'=1
;
;2003/08/20 - Change le nom des procedures appelees par refstar pour qu'il n'y ait pas
; de conflit avec la procedure imexam.pro qui utilise les memes noms mais pour du code different.
;
;2003/08/20 - Enleve l'affichage des 4 fenetres inutiles.
;
;2002/05/11 - Gere maintenant les images plus grandes que le display.
; Gere les differences dans le header pour le temps d'exposition de MONICA et LORAL.
;
;2002/05/27 - Modifie cieleur pour afficher les images plus grandes que MONICA
; sans que ca prenne des heures de calcul.
;
;2002/11/07 - Remarque au cas ou les psf sont mal fittees. Il faut jouer avec les
; bornes limites du fit dans la procedure refstar_moffatfit, voir les lignes suivantes:
; parinfo(1).limits(0) = parin[1] - 1.0*parin[4] (on pourrait agrandir a 1.5 ou 2.0 au lieu de 1.0)
; parinfo(1).limits(1) = parin[1] + 1.0*parin[4]
;
;2002/11/18 - On peut maintenant passer un fichier de clics de format nom_de_l_image,
; clic_x,clic_y

print, 'Procedure refstar, compare la photometrie d''une etoile dans plusieurs images.'
print

resolutionx = 1280
resolutiony = 1024

dimn = n_elements(images)

print,'Le nom de chaque image reduite:'
for i=0,dimn-1 do begin
	print, images[i]
endfor

ref_posx = fltarr(dimn)
ref_posy = fltarr(dimn)
ref_fwhm = fltarr(dimn)
ref_ampli = fltarr(dimn)
ref_beta = fltarr(dimn)
ref_flux = fltarr(dimn)
ref_ciel = fltarr(dimn)
xclic = fltarr(dimn)
yclic = fltarr(dimn)

;Etape 1. Trouver la position de l'etoile dans chaque image. Deux possibilites,
;soit on clique sur l'etoile pour chaque image, soit on automatise la detection
;de l'etoile. On pourrait demander de cliquer sur la premiere image puis ensuite
;automatiser la detection de la meme etoile sur les autres images.


;goto, cestbon
;D'abord, on verifie si un fichier de clics existe deja...
clic_existe = 0
if keyword_set(fichier_clic) then begin
	;Le fichier existe-t-il? On teste...
	if file_test(fichier_clic) then begin
		;On lit le fichier qui existe...
		readcol, fichier_clic, format='(A,F,F)', comment='#', imagelu, xcliclu, ycliclu
		;On verifie qu'il contient les memes images a cliquer
		if dimn eq n_elements(imagelu) then begin
			;On ne compare que le nom du fichier, pas celui du path entier
			;extrait le path des noms d'images
			imapref = images
			imasuff = images
			for im=0, n_elements(images)-1 do begin
				ima = images[im]
				otot = strsplit(ima,'/\',/extract)
				imasuff[im] = otot[n_elements(otot)-1]
				imapref[im] = strsplit(ima, imasuff[im], /extract,/regex)
			endfor
			for im=0, n_elements(imagelu)-1 do begin
				imlue = imagelu[im]
				otot = strsplit(imlue,'/\',/extract)
				imagelu[im] = otot[n_elements(otot)-1]
			endfor
			comparaison = strcmp(imagelu, imasuff,/fold_case)
			if mean(comparaison) eq 1.0 then begin
				clic_existe = 1
				print, 'Les clics sont enregistres dans le fichier '+fichier_clic
			endif
		endif
	endif
endif
;cestbon:
;clic_existe = 0

;On fait une boucle de toutes les images pour y cliquer l'etoile.
for imageencours=0,dimn-1 do begin

	;Trouvons le facteur de grossissement pour afficher l'image
	imagezoom1 = readfits(images[imageencours],entete,/silent)
	szima = size(imagezoom1)
	if szima(1) gt resolutionx*0.67 or szima(2) gt resolutiony*0.67 then begin
		;Image plus grande que le display --> il faut reduire la resolution (zoom < 1)
		;zoom = 1.0 / ceil(float(szima(1)) / (resolutionx*0.67))
		;print, zoom
		;Rebin l'image initiale
		;image = congrid(imagezoom1,zoom*szima(1),zoom*szima(2))
		zoom = 1.0 / (float(szima(1)) / (resolutionx*0.67))
		print, zoom
		;Rebin l'image initiale
		image = congrid(imagezoom1,zoom*szima(1),zoom*szima(2))
	endif else begin
		zoom = 1
		image = imagezoom1
	endelse
	print, 'zoom =', zoom
	;goto, fin
	window, 1, xsize=zoom*szima(1), ysize=zoom*szima(2), xpos=0, ypos=0, $
		title=strcompress('Position de l''etoile de reference dans ' + $
		images[imageencours] + ', ZOOM = ' + string(zoom))
	;Calculons la moyenne pour bien afficher l'image avec contraste
	moyenne = median(imagezoom1)
	largeur = median(abs(imagezoom1 - moyenne))/.69
	tvcm, image, z1=moyenne-2*largeur, z2=moyenne+15*largeur, silent=1,nb=!d.table_size-1

	if clic_existe then begin
		;Les clics ont deja ete fait precedemment
		xclic[imageencours] = xcliclu[imageencours]
		yclic[imageencours] = ycliclu[imageencours]
		print, 'Position deja cliquee: ', xclic[imageencours], yclic[imageencours]
	endif else begin
		;On determine la position grossiere du pic de l'etoile en cliquant
		print, 'Cliquez sur l''etoile de reference.'
		cursor, xcur, ycur, /normal
		xclic[imageencours] = xcur*szima[1]
		yclic[imageencours] = ycur*szima[2]
		print, 'Position cliquee: ', xclic[imageencours], yclic[imageencours]
	endelse

	;Fait un premier raffinement de la position du pic avec centroid
	;CNTRD, imagezoom1, xclic, yclic, xcen, ycen, 5.0, /debug;/SILENT
	;NON, ne fait pas de raffinement car CNTRD retourne xcen ou ycen
	;=NAN si il y a des NAN pres du pic, alors ca fait plante moffafit
	xcen = xclic[imageencours]
	ycen = yclic[imageencours]

	;Effectue le fit radial d'une fonction de Moffat
	parametre = fltarr(10)
	parametre[0] = moyenne 		;- le niveau du ciel pres de l'etoile
	parametre[1] = 50.0*largeur	;- l'amplitude au centre de la fonction (Io)
	parametre[2] = xcen		;- la position x du centre
	parametre[3] = ycen		;- la position y du centre
	parametre[4] = 3.0		;- le parametre R de la fonction moffatienne (FWHM)
	parametre[5] = 3.3		;- le parametre beta de la fonction moffatienne (positif et = 3-3.4)
	parametre[6] = largeur		;- niveau de bruit du ciel a un sigma
	parametre[7] = 6.0		;- le rayon sur lequel fitter la courbe
	parametre[8] = 4.0		;- le rayon sur lequel integrer le flux (~2*fwhm)
	parametre[9] = 0.0		;- le flux integre sous la courbe
	if keyword_set(resolelement) then begin
		parametre[4] = resolelement
		parametre[7] = 3*parametre[4]
		parametre[8] = 2*parametre[4]
	endif
;	parout = refstar_moffatfit(imagezoom1, parametre, beta=3.3, $
;			graphique=[resolutionx*0.67,resolutiony*0.75, $
;			resolutionx*0.33,resolutiony*0.25], cielsous=float(sxpar( entete ,'SKYVAL')) )
	parout = refstar_moffatfit(imagezoom1, parametre, beta=3.3, $
			cielsous=float(sxpar( entete ,'SKYVAL')) )

	;Affiche la position donnee par le fit
	tvcircle, 10*zoom, parout[2]*zoom, parout[3]*zoom
	;remplit le vecteur des resultats
	ref_flux[imageencours] = parout[9]
	ref_ampli[imageencours] = parout[1]
	ref_posx[imageencours] = parout[2]
	ref_posy[imageencours] = parout[3]
	ref_fwhm[imageencours] = parout[4]
	ref_beta[imageencours] = parout[5]

	;Determination du temps d'exposition. Le header de MONICA est ITIME, celui de LORAL est EXPTIME
	if sxpar( entete ,'ITIME') eq 0 then begin
		temps_expo = float(sxpar( entete ,'EXPTIME'))
	endif else begin
		temps_expo = float(sxpar( entete ,'ITIME'))
	endelse

;	;Affiche la courbe du flux en fonction des images
;	window, 3, xsize=resolutionx*0.33, ysize=resolutiony*0.25, $
;		xpos=resolutionx*0.67, ypos=resolutiony*0.00, $
;		title='Flux integre pour chaque image (ADU/sec)'
;	numeroimage = indgen(imageencours+1)+1
;	ref_flux[imageencours] = ref_flux[imageencours]/temps_expo
;	ref_flux_plot = ref_flux[0:imageencours]
;	;plot, numeroimage, ref_flux_plot, psym=6, xrange=[0,dimn+1]
;
;	;Affiche la courbe du FWHM en fonction des images
;	window, 4, xsize=resolutionx*0.33, ysize=resolutiony*0.25, $
;		xpos=resolutionx*0.67, ypos=resolutiony*0.25, $
;		title='FWHM pour chaque image'
;	numeroimage = indgen(imageencours+1)+1
;	ref_fwhm_plot = ref_fwhm[0:imageencours]
;	;plot, numeroimage, ref_fwhm_plot, psym=6, xrange=[0,dimn+1], $
;	;	yrange=[min(ref_fwhm_plot), max(ref_fwhm_plot)]
;
;	;Affiche le niveau du ciel soustrait en fonction des images
;	window, 5, xsize=resolutionx*0.33, ysize=resolutiony*0.25, $
;		xpos=resolutionx*0.67, ypos=resolutiony*0.50, $
;		title='Niveau du ciel pour chaque image (ADU/sec)'
;	numeroimage = indgen(imageencours+1)+1
;	ref_ciel[imageencours] = float(sxpar( entete ,'CIELSOUS'))/temps_expo
;	ref_ciel_plot = ref_ciel[0:imageencours]
;	;plot, numeroimage, ref_ciel_plot, psym=6, xrange=[0,dimn+1], $
;	;	yrange=[min(ref_ciel_plot), max(ref_ciel_plot)]

endfor

;Construit les vecteurs de deltas x et y
deltax = fltarr(dimn)
deltay = fltarr(dimn)
if keyword_set(absolu) then begin
	for i=0,dimn-1 do begin
		deltax[i] = ref_posx[i]
		deltay[i] = ref_posy[i]
	endfor
endif else begin
	for i=0,dimn-1 do begin
		;deltax[i] = -round(ref_posx[i] - ref_posx[0])
		;deltay[i] = -round(ref_posy[i] - ref_posy[0])
		deltax[i] = -(ref_posx[i] - ref_posx[0])
		deltay[i] = -(ref_posy[i] - ref_posy[0])
	endfor
endelse

;Enregistre les clics dans un fichier de clics s'il n'existe pas deja
if keyword_set(fichier_clic) and not clic_existe then begin
	openw,unite,fichier_clic, /get_lun
	for j = 0, n_elements(images)-1 do begin
		printf, unite, FORMAT='(A,F,F)', images[j], xclic[j], yclic[j]
	endfor
	close,unite
	free_lun, unite
endif


fin:
print,'Fin de la procedure refstar.'

wdelete

end











function refstar_moffatfit, image, param, beta=beta, graphique=graphique, cielsous=cielsous

;Cette fonction fait un fit radial moffat du profil d'une etoile
;dans image et retourne les parametres du meilleur fit.
;
;Voici les parametres a fournir
;
;param[0] - le niveau du ciel pres de l'etoile
;param[1] - l'amplitude au centre de la fonction (Io)
;param[2] - la position x du centre
;param[3] - la position y du centre
;param[4] - le parametre R de la fonction moffatienne (FWHM)
;param[5] - le parametre beta de la fonction moffatienne (positif et = 3-3.4)
;param[6] - niveau de bruit du ciel a un sigma
;param[7] - le rayon sur lequel fitter la courbe
;param[8] - le rayon sur lequel integrer le flux (~2*fwhm)
;param[9] - le flux integre sous la courbe
;
;keyword beta - si non nul, contraint beta a la valeur donnee.
;				si vecteur, limite beta entre les deux valeurs.
;keyword graphique - si non nul, le fit est affiche graphiquement
;					 = vecteur de 4 champs = [xpos,ypos,xsize,ysize]
;

;Fabrique la petite image mini qui contient les pixels a fitter
x1 = round(param[2] - param[7])
x2 = round(param[2] + param[7])
y1 = round(param[3] - param[7])
y2 = round(param[3] + param[7])
;print, x1,x2,y1,y2
;print, param[2], param[3], param[7]
;Fait une sous-image (mini) a partir de image[x1:x2,y1:y2] en checkant les marges
sousimage, image, mini, x1, x2, y1, y2, dx1, dy1
;Trouve la taille de l'image mini
sz = size(mini)
dimx = sz(1)
dimy = sz(2)
;Change les coordonnees du centre selon la nouvelle image mini
xminiin = param[2] - x1 - dx1
yminiin = param[3] - y1 - dy1
;print, x1, x2, y1, y2, dx1, dy1
;print, xminiin, yminiin
;mpfit2dfun pose probleme s'il y a des NAN, donc met a zero
for i=0, dimx-1 do begin
	;print, mini[i,*]
	for j=0, dimy-1 do begin
		if NOT FINITE(mini[i,j]) then mini[i,j] = mean(mini,/nan)
	endfor
endfor

;On prepare les x et y a passer a la fonction mpfit2dfun
xr = indgen(dimx)
yc = indgen(dimy)
xin = xr # (yc*0 + 1)
yin = (xr*0 + 1) # yc
zerr = fltarr(dimx*dimy)
zerr[*] = param[6]

;On initialise les parametre d'entree du fit
parin = dblarr(6)
parin[0] = param[1]		;Amplitude
parin[3] = param[4]		;fwhm
parin[4] = param[5]		;beta
parin[5] = param[0]		;ciel
parin[1] = xminiin		;x
parin[2] = yminiin		;y

;On impose les contraintes aux parametres du fit
parinfo = replicate({value:0.D, fixed:0, limited:[0,0], $
					limits:[0.D,0], mpminstep:0.D, mpmaxstep:0.D}, 6)
;fwhm plus grand que fwhm guess/2 et plus petit que fwhm guess*2
parinfo(3).limited(0) = 1
parinfo(3).limits(0) = parin[3]/2.0
parinfo(3).limited(1) = 1
parinfo(3).limits(1) = parin[3]*2.0
;Amplitude positive
parinfo(0).limited(0) = 1
parinfo(0).limits(0) = 0.0
;position a moins de deux fwhm du clic
;en X
parinfo(1).limited(0) = 1
parinfo(1).limits(0) = parin[1] - 2.0*parin[4]
parinfo(1).limited(1) = 1
parinfo(1).limits(1) = parin[1] + 2.0*parin[4]
;parinfo(1).mpminstep = 0.01
;parinfo(1).mpmaxstep = 0.1
;en Y
parinfo(2).limited(0) = 1
parinfo(2).limits(0) = parin[2] - 2.0*parin[4]
parinfo(2).limited(1) = 1
parinfo(2).limits(1) = parin[2] + 2.0*parin[4]
;parinfo(2).mpminstep = 0.01
;parinfo(2).mpmaxstep = 0.1
if keyword_set(beta) then begin
	if n_elements(beta) eq 2 then begin
		;beta entre beta[0] et beta[1]
		print, 'il y a deux limites a beta:', beta[0], beta[1]
		parinfo(4).limited(0) = 1
		parinfo(4).limits(0) = beta[0]
		parinfo(4).limited(1) = 1
		parinfo(4).limits(1) = beta[1]
	endif else begin
		if n_elements(beta) eq 1 then begin
			;beta = fixe
			parin[4] = beta
			parinfo(4).fixed = 1
		endif
	endelse
endif

;On fait le fit moffatien grace a mpfit2dfun disponible sur le web
parout = mpfit2dfun('refstar_moffatradial',xin,yin,mini,zerr,parin, parinfo=parinfo, quiet=1)

;On retransforme la position dans les coordonnees de l'image originale
xminiout = parout[1]
yminiout = parout[2]
parout[1] = x1 + parout[1]
parout[2] = y1 + parout[2]

;Calcule le flux total sous la courbe
param[9] = refstar_moffatflux(parout[0],parout[3],parout[4],param[8])	;flux integre

;Met les parametres a jour
param[0] = parout[5]	;- le niveau du ciel pres de l'etoile
param[1] = parout[0]	;- l'amplitude au centre de la fonction (Io)
param[2] = parout[1]	;- la position x du centre
param[3] = parout[2]	;- la position y du centre
param[4] = parout[3]	;- le parametre R de la fonction moffatienne (FWHM)
param[5] = parout[4]	;- le parametre beta de la fonction moffatienne (positif et = 3-3.4)

;Affiche le fit a l'ecran en appelant la procedure printfit
if keyword_set(graphique) then begin
	refstar_printmoffat, mini, xminiout, yminiout, graphique, param, cielsous=cielsous
endif

print,strcompress('Fit du clic a la position: '+string(parout[1])+', '+string(parout[2]))
return,param

end








FUNCTION refstar_moffatradial, x, y, param

;Fonction moffatienne radiale
;
;param[0] = Io		Amplitude
;param[1] = xo		Position x du centre
;param[2] = yo		Position y du centre
;param[3] = Ro		Rayon caracteristique
;param[4] = beta	Exposant beta
;param[5] = Iciel	Niveau du ciel
;
;r = sqrt( (x-xo)^2 + (y-yo)^2 )
;z = Iciel + Io * ( (1 + (r/Ro)^2)^(-beta) )

r = sqrt( (x-param[1])^2 + (y-param[2])^2 )
z = param[5] + param[0] * ( (1 + (r/param[3])^2)^(-param[4]) )
;z = param[0] * ( (1 + (r/param[3])^2)^(-param[4]) )

return, z
END




function refstar_moffatflux, I0, Ro, beta, rmax

;Il ne faut pas soustraire le niveau constant du ciel ici car
;Io represente l'amplitude au-dessus du niveau du ciel. Ce raisonnement
;fonctionne si le ciel a ete fitte precedemment. Sinon, la fonction
;d'etalement n'etait pas une moffatienne.

dr = 0.001
r1 = 0.0
r2 = r1 + dr
I = 0.0
repeat begin
	I = I + dr*3.14159265*I0* ((1.0 + (r2/Ro)^2.0)^(-beta) + (1.0 + (r1/Ro)^2.0)^(-beta))
	r1 = r2
	r2 = r1 + dr
	;print, I
endrep until r1 ge rmax

return, I

end



pro refstar_printmoffat, echantillon, xmini, ymini, fenetre, param, cielsous=cielsous

;param[0] - le niveau du ciel pres de l'etoile
;param[1] - l'amplitude au centre de la fonction (Io)
;param[2] - la position x du centre
;param[3] - la position y du centre
;param[4] - le parametre R de la fonction moffatienne (FWHM)
;param[5] - le parametre beta de la fonction moffatienne (positif et = 3-3.4)
;param[6] - niveau de bruit du ciel a un sigma
;param[7] - le rayon sur lequel fitter la courbe
;param[8] - le rayon sur lequel integrer le flux (~2*fwhm)
;param[9] - le flux integre sous la courbe

if not keyword_set(cielsous) then cielsous=0.0

fenetre_initiale = !d.window

szsample = size(echantillon)
dimx = szsample[1]
dimy = szsample[2]
posr = fltarr(szsample[1]*szsample[2])
valr = fltarr(szsample[1]*szsample[2])

h = 0
for i=0,dimx-1 do begin
	for j=0,dimy-1 do begin
		posr[h] = sqrt((xmini-i)^2 + (ymini-j)^2)
		valr[h] = echantillon[i,j]
		h = h + 1
	endfor
endfor

window, fenetre_initiale+1, xsize=fenetre[2], ysize=fenetre[3], $
	xpos=fenetre[0], ypos=fenetre[1], $
	title='Fit moffatien du profil de l''etoile.'
;Genere la courbe du fit moffatien
moffatr = findgen(20*max(posr))/10.0
moffatz = param[0] + param[1]*(1 + (moffatr/param[4])^2)^(-param[5])
;Pour ajouter le niveau du ciel soustrait a la reduction...
moffatz = moffatz + cielsous
valr = valr + cielsous

;plot, moffatr, moffatz, xrange=[0,param[7]+1], yrange=[min(valr),max(moffatz)]
;oplot, posr, valr, psym=6

;XYOUTS, 0.4, 0.82, strcompress('Position'+string(param[2])+','+string(param[3])),normal=1
;XYOUTS, 0.4, 0.75, strcompress('FWHM (pixels) = '+string(param[4])),normal=1
;XYOUTS, 0.4, 0.68, strcompress('Amplitude max = '+string(param[1])),normal=1
;XYOUTS, 0.4, 0.61, strcompress('Exposant beta = '+string(param[5])),normal=1
;XYOUTS, 0.4, 0.54, strcompress('Flux integre = '+string(param[9])),normal=1

wset, fenetre_initiale

end






pro sousimage, image, sousima, x1, x2, y1, y2, dx1, dy1

dx1 = 0
dy1 = 0
szima = size(image)
dimx = szima[1]
dimy = szima[2]

if x1 lt 0 then begin
	dx1 = 0-x1
	x1 = 0
endif
if x2 gt dimx-1 then x2 = dimx-1
if y1 lt 0 then begin
	dy1 = 0-y1
	y1 = 0
endif
if y2 gt dimy-1 then y2 = dimy-1

sousima = image[x1:x2,y1:y2]

end










