function simon_detectetype, entete

;Cette fonction detecte le type d'image dont il est question: dark, flat on, flat off ou objet.

titre = sxpar(entete, 'OBJECT')

;Conditions pour que ce soit un dark:
;1 - C'est ecrit dark (priorite 2)
;2 - Il y a le blank installe en mode imagerie (priorite 1)
;3 - Les temps de pose des darks correspondent aux temps de pose des 'objets' (priorite 4)
;4 - C'est fait en fin de nuit ou au debut de nuit ou durant la nuit s'il n'y a pas
;    d'autres objets durant cette nuit. (priorite 3)

;Conditions pour que ce soit un objet:
;1 - Ce n'est ni ecrit dark, ni flat, ni test 
;2 - L'image est prise en mode imagerie, pas spectro ou autre
;3 - Detection de sources a plus de 10 sigmas
;4 - C'est fait durant la nuit

;Conditions pour que ce soit un flat on/off:
;1 - C'est ecrit flat on/off
;2 - C'est en mode imagerie avec filtre, pas blank
;3 - Il y a symetrie on/off avec les memes temps de pose
;4 - C'est fait en fin de nuit ou au debut de nuit ou durant la nuit s'il n'y a pas
;    d'autres objets durant cette nuit.
;5 - Les coordonnees du telescope sont vers l'ecran de flat (pas toujours a jour)

;print, titre

;Detecte les darks si leur nom est dark et si les roues sont a blank
isdark = stregex(titre, '(.ark)|(d.rk)|(da.k)|(dar.)', /fold_case)
if isdark ne -1 then begin
	;print, 'Il y a dark ecrit comme titre'
	filtre = simon_detectefiltre(entete)
	if strcmp(filtre, 'blank') eq 1 then begin
		;print, 'Les roues sont en mode imagerie avec un blank'
		;print, 'C''est un dark avec le filtre ',filtre
		return, 'dark'
	endif
endif

;Detecte les flats si leur nom est flat et si les roues sont en mode imagerie
isflat = stregex(titre, '(.lat)|(f.at)|(fl.t)|(fla.)', /fold_case)
if isflat ne -1 then begin
	ison = stregex(titre, '(on)|(allum)|(alum)|(ouver)', /fold_case)
	filtre = simon_detectefiltre(entete)
	if ison ne -1 then begin
		if strcmp(filtre, '-1') ne 1 then begin
			;print, 'Les roues sont en mode imagerie, filtre ',filtre
			;print, 'C''est un flat ON (lumiere allumee), filtre ',filtre
			return, 'flaton'
		endif
	endif
	isoff = stregex(titre, '(of.)|(etein)|(etain)|(ferm)', /fold_case)
	if isoff ne -1 then begin
		if strcmp(filtre, '-1') ne 1 then begin
			;print, 'Les roues sont en mode imagerie, filtre ',filtre
			;print, 'C''est un flat OFF (lumiere eteinte), filtre ',filtre
			return, 'flatoff'
		endif
	endif
endif

;Detecte les objets si les roues sont en mode imagerie
filtre = simon_detectefiltre(entete)
isimagerie = stregex(filtre, '(I)|(J)|(H)|(Ks)|(H2)|(FeII)|(BrGamma)|(PaBeta)')
if isimagerie ne -1 then begin
	;print, 'Les roues sont en mode imagerie, filtre ',filtre
	return, 'objet'
endif


return, '-1'

end
