pro simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, $
	nomCIEL=nomCIEL, nomBADPIX=nomBADPIX, selon_temps=selon_temps, $
	ciel_plus_crepuscule=ciel_plus_crepuscule, garde_ciel=garde_ciel, $
	soustrait_ligne_mediane=soustrait_ligne_mediane, $
	soustrait_colonne_mediane=soustrait_colonne_mediane	

;nomOBJbrut : une liste des noms des images brutes
;nomOBJredu : une liste des noms de ces images une fois reduites
;nomFLAT : nom du flatfield applicable a ces images
;nomDARK : nom du dark applicable a ces images
;nomCIEL (option) : nom de l'image du ciel deja construite pour soustraire le ciel 
;		    des images. Si vous ne voulez PAS soustraire le ciel, ne specifiez
;		    pas d'image.
;nomBADPIX (option) : nom de l'image des mauvais pixels de SIMON. Si vous ne l'avez pas
;                     alors ne la specifiez pas.
;selon_temps (option) : si selon_temps=1 alors le ciel sera soustrait avec l'image
;			nomCIEL (si elle est aussi specifiee) mais en ne se fiant que
;			sur le temps de pose de l'image, pas sur le niveau median du
;			ciel qu'on y trouve. Autrement dit, utilisez cette option pour
;			soustraire le ciel d'une source etendue dont l'evaluation du
;			niveau median du ciel n'est pas un bon indicateur du vrai ciel
;			present.
;ciel_plus_crepuscule (option) : si ciel_plus_crepuscule=1 et que nomCIEL est defini
;			alors la soustraction du ciel sera optimisee dans le cas ou
;			on suspecte que le ciel est plus brillant a cause de lumiere
;			diffusee plutot qu'a cause de la brillance des raies OH- du
;			ciel noir. C'est le cas au crepuscule ou lorsque la Lune est
;			proche du champ d'observation.
;garde_ciel (option) : Si garde_ciel=1 alors le ciel n'est pas soustrait des images.
;soustrait_ligne_mediane (option): Si soustrait_ligne_mediane=1 alors on soustrait a
;			l'image reduite la valeur mediane des lignes de l'image. Ceci
;			peut corriger les problemes quand il existe une structure
;			verticale systematique a la jonction des quadrants.


;Le flat, le dark, le ciel et le badpix sont lus
imDARK = readfits(nomDARK, /silent)
imFLAT = readfits(nomFLAT, /silent)
if keyword_set(nomCIEL) eq 1 then begin
	imCIEL = readfits(nomCIEL, entete_ciel, /silent)
endif
if keyword_set(nomBADPIX) eq 1 then begin
	imBADPIX = readfits(nomBADPIX, /silent)
endif


;La boucle de reduction sur chacune des images brutes de l'objet (nomOBJbrut)
for i = 0, n_elements(nomOBJbrut)-1 do begin
	imOBJtmp = readfits(nomOBJbrut[i],entete,/silent)
	print, strcompress('Lit et reduit ' + nomOBJbrut[i])
	;Division par le nombre d'images coadditionnees et par le nombre de lectures
	imOBJtmp = imOBJtmp / (sxpar(entete, 'NCOA') * sxpar(entete, 'NLEC'))
	;Correction pour la non-linearite - NON-IMPLEMENTE AVEC SIMON ENCORE
	;print, '     *Corrige pour la reponse non-lineaire du detecteur.'
	;imOBJtmp = nonlincorr(imOBJtmp)
	;Soustraction du dark
	print, '     *Soustrait le dark.'
	imOBJtmp = imOBJtmp - imDARK
	;Division par le flat
	print, '     *Divise par le flat.'
	imOBJtmp = imOBJtmp / imFLAT
	;Muliplication par l'image des mauvais pixels (si elle est fournie)
	if keyword_set(nomBADPIX) eq 1 then begin
		print, '     *Masque les mauvais pixels du detecteur.'
		imOBJtmp = imOBJtmp * imBADPIX
	endif
	;Elimination des pixels satures
	print, '     *Masque les pixels satures (>30000adu).'
	imOBJtmp = enleve_pixelssatures(imOBJtmp, 30000)
	;Soustraction du ciel
	case 1 of
		keyword_set(nomCIEL) and keyword_set(selon_temps): begin
		;On soustrait le ciel mais au lieu de multiplier l'image du ciel par
		;la mediane du ciel de l'image a reduire, on se sert du temps d'exposition
		;des images comme facteurs multiplicatifs. EX: Si on observe une source
		;etendue diffuse, la mediane de l'image a reduire n'est pas une bonne mesure
		;du niveau du ciel, il faut alors utiliser une image du ciel et multiplier
		;par le rapport des temps de pose.
			print, '     *Soustrait le ciel selon les temps de pose du ciel et de l''image.'
			;Lit dans l'entete le flux en adu/sec moyen de l'image du ciel
			niveau = sxpar(entete, 'NIVCIEL')
			if not sxpar(entete,'TINT') then begin
				tlec = sxpar(entete,'TLEC')
				teff = sxpar(entete,'TEFF')
				nlec = sxpar(entete,'NLEC')
				tint = teff - tlec*(nlec-1)/2.0
				imOBJtmp = imOBJtmp - imCIEL*niveau*tint
			endif else begin
				imOBJtmp = imOBJtmp - (imCIEL*niveau*sxpar(entete,'TINT'))
			endelse
		end
		keyword_set(nomCIEL) and keyword_set(ciel_plus_crepuscule): begin
		;On soustrait le ciel plus une constante non-nulle determinee de facon
		;empirique et iterative en minimisant la deviation standard des residus
		;de soustraction. EX: sur les images prisent au crepuscule ou pres de la
		;lune.
			print, '     *Soustrait le ciel plus une constante et minimise l''ecart type.'
			imOBJtmp = fit_cielconstant(imOBJtmp, imCIEL)
		end
		keyword_set(nomCIEL) eq 0 or keyword_set(garde_ciel): begin
		;On ne soustrait pas le ciel du tout
			print, '     *Ne soustrait pas l''image du ciel!'

		end
		else: begin
		;On soustrait le ciel de facon conventionnelle
			print, '     *Soustrait le ciel de facon conventionnelle en se fiant sur le niveau du ciel dans l''image.'
			imOBJtmp = imOBJtmp - ( imCIEL * median(imOBJtmp) / median(imCIEL) )
		end	
	endcase
	;Correction pour les mauvais pixels? NON-IMPLEMENTE ENCORE
	;Correction pour les structures verticales - on soustrait la mediane des colonnes
	if keyword_set(soustrait_colonne_mediane) then begin
		print, '     *Soustrait la mediane des colonnes pour corriger les structures verticales.'     
		colonnemediane = fltarr(1024,1024)
		for j=0, 1023 do begin
			colonne = imOBJtmp[j,*]
			for iter=0, 2 do begin
				valmediane = median(colonne)
				ecartmedian = ecart_median(colonne)
				index = where((colonne - valmediane) ge 5*ecartmedian, nindex)
				if nindex ge 1 then colonne[index] = !values.f_nan
			endfor
			colonnemediane[j,*] = median(colonne)
		endfor
		imOBJtmp = imOBJtmp - colonnemediane
	endif
	;Correction pour les structures horizontales - on soustrait la mediane des lignes
	if keyword_set(soustrait_ligne_mediane) then begin
		print, '     *Soustrait la mediane des lignes pour corriger les structures horizontales.'     
		lignemediane = fltarr(1024,1024)
		for j=0, 1023 do begin
			ligne = imOBJtmp[0:511,j]
			for iter=0, 2 do begin
				valmediane = median(ligne)
				ecartmedian = ecart_median(ligne)
				index = where((ligne - valmediane) ge 5*ecartmedian, nindex)
				if nindex ge 1 then ligne[index] = !values.f_nan
			endfor
			lignemediane[0:511,j] = median(ligne)
			ligne = imOBJtmp[512:1023,j]
			for iter=0, 2 do begin
				valmediane = median(ligne)
				ecartmedian = ecart_median(ligne)
				index = where((ligne - valmediane) ge 5*ecartmedian, nindex)
				if nindex ge 1 then ligne[index] = !values.f_nan
			endfor
			lignemediane[512:1023,j] = median(ligne)		
		endfor
		imOBJtmp = imOBJtmp - lignemediane
	endif
	
	;Division par le temps d'exposition pour normaliser a une seconde d'exposition
	print, '     *Divise par le temps d''exposition.'
	if not sxpar(entete,'TINT') then begin
		tlec = sxpar(entete,'TLEC')
		teff = sxpar(entete,'TEFF')
		nlec = sxpar(entete,'NLEC')
		tint = teff - tlec*(nlec-1)/2.0
	endif else begin
		tint = sxpar(entete,'TINT')
	endelse
	imOBJtmp = imOBJtmp / tint
	;Ecriture de l'image scientifique reduite
	print, '     *Ecrit l''image reduite sur disque.'
	writefits, nomOBJredu[i], imOBJtmp, entete
endfor



end ;FIN DE LA PROCEDURE SIMON_OBJET
