pro simon_reducimagerie

goto, doreduc

;*********************************************************************************************
;Script de reduction des images prises avec SIMON en MODE IMAGERIE
;                                                    -------------
;Loic Albert - 20 aout 2003
;              12 septembre 2003
;	       26 septembre 2003
;	       8 avril 2004
;
;albert@astro.umontreal.ca
;
;Le package a ete concu pour fonctionner sur IDL5.4 (et plus recent) et utilise la librairie de
;procedures IDL Astrolib disponible sur le web. Le package fonctionne aussi bien sur systeme Unix
;que Windows (sauf la partie de reduction automatique qui ne fonctionne pas sur Windows pourl'instant)
;
;Voici la liste des procedures IDL utilisees:
;	simon_reducimagerie.pro
;	simon_badpix.pro
;	simon_dark.pro
;	simon_flat.pro
;	simon_ciel.pro
;	simon_objet.pro
;	imcorrel.pro
;	simon_dark_oao_pente.pro
;	refstar.pro
;	imsum.pro
;	filtreOS.pro
;	faitnoms.pro
;	ecart_median.pro
;	fit_cielconstant.pro
;	enleve_pixelssatures.pro
;	effaceNAN.pro
;	simon_automatique.pro
;	simon_detectetype.pro
;	simon_detectefiltre.pro
;	+la librairie IDL astrolib disponible sur http://idlastro.gsfc.nasa.gov/ftp/
;	 mais deja disponible sur le reseau astro a Montreal.
;	+la librairie mpfit pour la partie de combinaison des images et de cliquage
;	mpcurvefit.pro
;	mpfit2dfun.pro
;	mpfit2dpeak.pro
;	mpfitexpr.pro
;	mpfitfun.pro
;	mpfitpeak.pro
;	mpfit.pro
;*********************************************************************************************
;
;TABLE DES MATIERES
;------------------
;
;1-IMAGES DE CALIBRATION NECESSAIRES A LA REDUCTION DES IMAGES
;	A - Prendre de flats avec lampes allumees et lampes eteintes
;	B - Prendre des darks avec les memes temps de pose que vos images (scientifiques
;	    et de calibration)
;	C - (Optionnel) Prendre une sequence de flats pour fabriquer l'image des mauvais pixels
;   D - Avoir un bon logbook ecrit
;2-REDUCTION DES IMAGES DE CALIBRATION ET SCIENTIFIQUES
;	ETAPE 1 - Definir les repertoires ou sont lues vos donnees brutes et ecrites vos
;		  images reduites
;	ETAPE 2 - Fabrication d'une image des mauvais pixels de SIMON (optionel - il en
;                 existe deja une sur cyga)
;	ETAPE 3 - Fabrication des darks (mediane de plusieurs images brutes, etc)
;	ETAPE 4 - Fabrication des flats a partir des images de lampes allumees,
;		  lampes eteintes
;	ETAPE 5 - Reduction des images scientifiques en utilisant les darks et les flats
;		  fabriques dans les etapes precedentes. Ceci inclue la fabrication et la
;		  soustraction du ciel qui depend du type d'observation (source ponctuelle
;		  ou source etendue)
;	en developpement... ETAPE 6 - Combinaison des images en une image mosaique finale (pour un filtre
;                 donne, i.e. ca ne fait pas une image couleur!)
;
;
;
;
;
;
;
;
;
;
;1-IMAGES DE CALIBRATION NECESSAIRES A LA REDUCTION DES IMAGES
;=============================================================
;
; A - LES FLATS
; -------------
;
;  Prendre des domeflats au moins en debut de nuit, ou mieux, en debut et fin de nuit. Il faut
;  prendre suffisamment d'images pour avoir ~100000 ADU au total de niveau de ciel, avec environ
;  10000 ADU par image. Selon mon experience, les skyflats au crepuscule sont inutiles et
;  fonctionnent moins bien que les domeflats faits sur l'ecran blanc. Evidemment, il faut obtenir
;  des flats pour chaque filtre. Comme les flats ont tendance a changer au cours du temps, il faut
;  en prendre le plus souvent possible au cours de la mission.
;
;  La meilleure methode pour obtenir de bons flats est la suivante. Vous prenez des poses, d'abord
;  avec les lampes de flat allumees de la facon classique. Puis, immediatement apres, vous prenez
;  des poses (meme temps d'integration, NLEC, NCOADD, etc) mais avec les lampes eteintes et SIMON
;  dans le noir. NE METTEZ PAS le blank! Ainsi, la contribution du telescope, de la lumiere diffuse
;  ou de tout autre source, pourra etre soustraite.
;
;  Ce script de reduction suppose que c'est ce type de calibration que vous avez fait. Si ce n'est
;  pas le cas, ecrivez un nouveau script vous-meme!
;
;
; B - LES DARKS
; -------------
;
;  Prendre des darks pour chaque temps de pose que vous fait pour vos images scientifiques et
;  vos images de calibration, et ce pour chaque filtre utilise. Par exemple, je prend 25 darks
;  (coadditionnes) de 1sec, de 6sec et 10 darks de 45sec si j'ai fait des flats de 1sec et 6sec
;  et que mes images scientifiques sont toutes de 45sec.
;
;  Les darks sont les images prises avec le BLANK en place dans la roue de SIMON.
;
;
; C - LA SEQUENCE DE FLATS POUR CREER UNE IMAGE DES MAUVAIS PIXELS (OPTIONNEL)
; ----------------------------------------------------------------------------
;
;  Prendre une serie de domeflats de differents niveaux d'intensite dans un seul filtre. C'est
;  pour utiliser dans la fabrication d'une image des mauvais pixels. Il est preferable d'obtenir
;  des images d'au moins 5-6 niveaux d'intensite differents.
;
;  Une telle image a deja ete faite et vous la trouverez sur cyga (132.204.61.23) dans le
;  repertoire /home/simon/imageref/    user=visiteur  password=obs_omm
;
;
; D - LE LOGBOOK
; --------------
;
;  Ce script ne detecte pas automatiquement ni n'ordonne les images selon l'objet observe ou
;  le filtre utilise. Ca, c'est votre travail. C'est aussi un choix editorial. A qui faites-
;  vous le plus confiance? A l'intelligence artificielle ou a vous-meme?
;
;  C'est pourquoi je vous recommande d'avoir a votre disposition de bonnes notes prises lors
;  de votre mission. Vous les utiliserez frequemment dans ce script. Si jamais vous n'en avez
;  pas pris (Hey! C'est 4000$ par nuit claire a Megantic!), un logbook de secours est automa-
;  tiquement genere pour les observations SIMON a chaque nuit. On le trouve dans le repertoire
;  ou a ete lance MIR lors votre mission, c'est un fichier .log du nom de la date de vos
;  observations.
;
;
;2-REDUCTION DES IMAGES DE CALIBRATION ET SCIENTIFIQUES
;======================================================


;##############################################################################
; ETAPE 1 - DEFINITION DES PATHS POUR VOS IMAGES BRUTES, REDUITES ET VOS IMAGES
;           DE CALIBRATION. VOUS POUVEZ LANCER CE PROGRAMME SUR WINDOWS OU UNIX
;
; LA DEFINITION DES PATHS SIMPLIFIE BEAUCOUP LA VIE POUR LE RESTE DU PROGRAMME.
; ENSUITE, ON PEUT FORMER LE NOM DES IMAGES BRUTES, REDUITES OU DE CALIBRATION
; PLUS FACILEMENT. POUR CE FAIRE, DEUX FONCTION SIMPLES SONT APPELEES TRES SOU-
; VENT: faitnoms et filtreOS. VOICI UN EXEMPLE DE LA SYNTAXE DE CES DEUX FONC-
; TIONS.
;
; Si:
; 	path_imagesbrutes = '/home/loic/fits/'
; 	nom = faitnoms(path_imagesbrutes+'030212_','0031,0035,0100-0102','.fits')
; Alors:
; IDL> print, nom
; 	/home/loic/fits/030212_0031.fits /home/loic/fits/030212_0035.fits
;	/home/loic/fits/030212_0100.fits /home/loic/fits/030212_0101.fits
;	/home/loic/fits/030212_0102.fits
;
; *Notez que vous pouvez mettre des virgules pour deux fichiers non-consecutifs
; et des tirets pour une suite de fichiers.
;
; Alors que filtreOS est utile lorsque vous utilisez deux systemes d'exploita-
; tion differents et que vous ne voulez pas vous cassez la tete sur l'utilisa-
; tion des '/' (en unix) ou des '\' (en windows).
;
; EXEMPLE:
;
; nom = filtreOS(path_imagesbrutes+'nuit1\flatJ.fits')
;
; donne:
;	'/home/loic/fits/nuit1/flatJ.fits' Si vous etes en unix
;	'c:\simon\fits\nuit1\flatJ.fits' Si vous etes en windows
;
; pourvu que vous ayant prealablement defini:
;      path_imagesbrutes = '/home/loic/fits/' sous unix
;      path_imagesbrutes = 'c:\simon\fits\' sous windows
;
;##############################################################################

;Ici, modifiez les paths selon vos besoins
if strcmp(!version.os_family, 'unix',/fold_case) eq 1 then begin
		path_imagesbrutes = '/home/loic/fits/'
		path_imagesreduites = '/home/loic/reduc/'
		path_imagescalibration = '/home/loic/flats/'
endif else begin
	if strcmp(!version.os_family, 'Windows',/fold_case) eq 1 then begin
		path_imagesbrutes = 'c:\simon\fits\'
		path_imagesreduites = 'c:\simon\reduc\'
		path_imagescalibration = 'c:\simon\flats\'
	endif
endelse



;##############################################################################
; ETAPE 2 -   FABRICATION D'UNE IMAGE DES MAUVAIS PIXELS DU DETECTEUR DE SIMON.
; (Optionnel) CELA VA SERVIR DURANT TOUTE LA REDUCTION AFIN QUE VOS IMAGES FI-
;             NALES COMBINEES SOIENT EXEMPTES DE CES DEFAUTS.
;
; POUR REALISER CETTE ETAPE, VOUS AVEZ BESOIN D'UNE SERIE D'IMAGES DE DOMEFLATS
; PRISES A DIFFERENTS NIVEAUX DE FLUX EN SEQUENCE. PAR EXEMPLE, PRENDRE 7 IMA-
; GES EN ORDRE CROISSANT DE TEMPS D'EXPOSITION:3,6,9,12,15,18,21 SECONDES.
;
; SI VOUS N'AVEZ PAS PRIS CES IMAGES LORS DE VOTRE MISSION, VOUS POUVEZ REMPLA-
; CER PAR L'IMAGE D'UN TRES BON FLAT ET APPELER simon_badpix COMME DANS L'EXEM-
; PLE 2. SI VOUS ETES VRAIMENT PARESSEUX, UTILISEZ L'IMAGE DES MAUVAIS PIXELS
; QUI SE TROUVE SUR LE SITE WEB DE SIMON OU SUR LE REPERTOIRE
; /home/simon/imageref SUR cyga. (telnet 132.204.61.23 user=visiteur password=
;                                 obs_omm)
;##############################################################################
dobadpix:
for iteration = 1,1 do begin ;Modifiez les chiffres pour que le programme aille
			     ;a la bonne iteration, exemple iteration = 3,5 pour
			     ;effectuer les iteration 3 a 5. Dans l'exemple actuel,
			     ;il n'y a que 2 boucles possibles: 1 et 2.
	print, 'Fabrique l''image des mauvais pixels #', iteration

	;PREMIER EXEMPLE POUR FABRIQUER UNE IMAGE DES MAUVAIS PIXELS DE SIMON
	;--- AVEC UNE SERIE DE DOMEFLATS PRIS A DIFFERENTS TEMPS DE POSE:
	;    ALORS, TROIS IMAGES SONT CREEES. L'IMAGE DE LA PENTE (FLUX PAR SECONDE)
	;    L'IMAGE DE L'ORDONNEE A L'ORIGINE (FLUX POUR TEXP = 0SEC) ET
	;    L'IMAGE DES MAUVAIS PIXELS: BON PIXEL = 1, MAUVAIS PIXEL = NAN
	; *** CA PEUT PRENDRE 30-45 MINUTES DE CALCULS MAIS C'EST FIABLE.
	if iteration eq 1 then begin
		nomFLATS = faitnoms(path_imagesbrutes+'030212_', '0031-0036','.fits')
		nomOAO = filtreOS(path_imagescalibration+'oao030212.fits')
		nomPENTE = filtreOS(path_imagescalibration+'pente030212.fits')
		nomBADPIX = filtreOS(path_imagescalibration+'badpix030212.fits')
		;simon_badpix est une procedure a part. C'est elle qui fait tout le
		;travail. Vous pouvez l'ouvrir pour comprendre ce qu'elle fait exactement.
		simon_badpix, nomFLATS, nomBADPIX, nomOAO=nomOAO, nomPENTE=nomPENTE
	endif

	;DEUXIEME EXEMPLE
	;--- CETTE FOIS, IL FAUT SIMPLEMENT ENTRER LE NOM D'UN BON FLAT. L'IMAGE
	;    DES MAUVAIS PIXELS SERA CREEE A PARTIR DE CE SEUL FLAT.
	; *** C'EST TRES RAPIDE!
	if iteration eq 2 then begin
		nomFLATS = filtreOS(path_imagescalibration+'flatJ030212.fits')
		nomBADPIX = filtreOS(path_imagescalibration+'badpix030212.fits')
		simon_badpix, nomFLATS, nomBADPIX
	endif
endfor



;##############################################################################
; ETAPE 3 - FABRICATION DES DARKS - C'EST-A-DIRE DES IMAGES PRISENT AVEC SIMON
;           COMPLETEMENT DANS LE NOIR ET AVEC LE BLANK EN PLACE DANS UNE DES
;           ROUES A FILTRES.
; RAPPEL: UN DARK CONSTITUE LE BRUIT DE LECTURE DU DETECTEUR LORSQUE AUCUNE
;         LUMIERE NE L'ILLUMINE. C'EST L'EQUIVALENT DU 'BIAS' POUR LES GENS
;         FAMILIERS AVEC L'OBSERVATION OPTIQUE AVEC LES CCD. MAIS A LA DIFFE-
;         RENCE DES BIAS, UN DARK DEPEND DU TEMPS D'EXPOSITION QUE L'ON FAIT.
;         EN EFFET, 8 REGIONS 'CHAUDES' APPARAISSENT SUR LE DETECTEUR DE SIMON
;         ET L'INTENSITE DE CES TACHES EST PROPORTIONNELLE AU TEMPS D'INTEGRA-
;         TION. C'EST POURQUOI, IL FAUT PRENDRE DES DARKS CORRESPONDANTS AUX
;         MEMES TEMPS D'INTEGRATION UTILISES LORS D'UNE MISSION. PAR EXEMPLE,
;         SI VOS IMAGES SCIENTIFIQUES ET DE CALIBRATION ONT DES TEMPS DE POSE
;         DE 0,3,5,10,20,30,60,90 SECONDES, IL FAUT QUE VOUS PRENIEZ UNE SERIE
;         DE DARKS POUR CHACUN DE CES TEMPS. VOUS REFERER AU MANUEL DE SIMON
;         CAR IL EXISTE DES SCRIPTS POUR CE FAIRE.
;##############################################################################
dodark:
for iteration = 1, 10 do begin
	print, 'Reduit le set de darks #',iteration

	;VOICI 9 EXEMPLES DE DARKS. POUR CHAQUE TEMPS DE POSE, IL Y A 3 IMAGES. MAIS
	;CHAQUE IMAGE REPRESENTE UNE COADDITION DE PLUSIEURS IMAGES LORS DES OBSERVATIONS.
	;POUR LES DARKS, VOUS POUVEZ MELANGER DES DARKS PRIS AVEC DIFFERENTS COADD OU
	;NFIC, POURVU QUE VOUS AYEZ LES MEMES TINT ET MEMES NLEC.
	if iteration eq 1 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0176-0178','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_6sec.fits')
		;simon_dark est la procedure qui fait tout le travail. Pour comprendre
		;n'hesitez-pas a l'ouvrir, il y a des commentaires!
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 2 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0179-0181','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_15sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 3 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0182-0184','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_20sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 4 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0185-0187','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_25sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 5 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0188-0190','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_30sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 6 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0191-0193','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_40sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 7 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0194-0196','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_45sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 8 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0197-0199','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_60sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	if iteration eq 9 then begin
		nomDARKbrut = faitnoms(path_imagesbrutes+'030327_', '0200-202','.fits')
		nomDARKfinal = filtreOS(path_imagescalibration+'dark_blank_120sec.fits')
		simon_dark, nomDARKbrut, nomDARKfinal
	endif
	;VOICI UN EXEMPLE PAS MAL PLUS TORDU DANS LE CAS OU VOUS N'AURIEZ PAS PRIS DE
	;DARK LORS DE VOTRE MISSION. LORS DES DIFFERENTES MISSIONS D'INGENIERIE, ON A
	;CONSTATE QUE LES DARKS SONT TRES STABLES D'EPOQUE EN EPOQUE. ON MISE LA-DESSUS
	;POUR FABRIQUER DEUX IMAGES CORRESPONDANT A LA PENTE DES DARKS (FLUX/SEC) ET
	;A L'ORDONNEE A L'ORIGINE DES DARKS (FLUX) AFIN DE RECONSTRUIRE DES DARKS DE
	;N'IMPORTE QUEL TEMPS DE POSE DESIRE. LES IMAGES dark_oao et dark_pente SONT
	;DISPONIBLES SUR L'ORDINATEUR DE CONTROLE DE SIMON, cyga, /home/simon/imageref/ 
	;OU SUR LE SITE WEB DE SIMON http://www.astro.umontreal.ca/~albert/simon_autre/simon2.html.
	if iteration eq 10 then begin
		nomDARKS = [filtreOS(path_imagescalibration+'dark_blank_30sec_030613.fits'), $
			filtreOS(path_imagescalibration+'dark_blank_60sec_030613.fits'), $
			filtreOS(path_imagescalibration+'dark_blank_120sec_030613.fits')]
		tint = [28.99,57.98,118.59]
		nomOAO = filtreOS(path_imagescalibration+'dark_oao_030613.fits')
		nomPENTE = filtreOS(path_imagescalibration+'dark_pente_030613.fits')
		;Construit les deux images de darks a partir de darks de diff. temps de pose
		simon_dark_oao_pente, nomDARKS, tint, nomOAO, nomPENTE
		;Ensuite, pour construire un dark d'un temps de pose voulu, il faut
		;faire dark(n sec) = dark_oao + dark_pente * n
	endif



endfor




;##############################################################################
; ETAPE 4 - FABRICATION DES FLATS - A PARTIR DES IMAGES AVEC LAMPES ALLUMEES ET
;           LAMPES ETEINTES.
;
; LA MEILLEURE METHODE POUR OBTENIR DE BONS FLATS EST DE PRENDRE DES IMAGES
; AVEC LE TELESCOPE POINTE SUR L'ECRAN DE FLATS AVEC LAMPES ALLUMEES. PUIS,
; IMMEDIATEMENT APRES, DE PRENDRE LE MEME NOMBRE D'IMAGES MAIS EN ETEIGNANT LES
; LUMIERES ET SANS RIEN CHANGE DU SETUP DE SIMON (MEME TEMPS DE POSE, MEME
; FILTRE, PAS DE BLANK!, ETC). AINSI, LA SOUSTRACTION DES DEUX ENSEMBLE D'IMA-
; GES SERA EXEMPTS DE LA POLLUTION THERMIQUE DU TELESCOPE, DES VARIATIONS DE
; LA LUMIERE DIFFUSE OU DE LA TEMPERATURE DU DETECTEUR, ETC.
;
; SI TOUTEFOIS, VOUS N'AVEZ PAS SUIVI CETTE METHODE, VOUS POUVEZ CREER VOS
; FLATS QUAND MEME (VOIR L'EXEMPLE 4) MAIS ILS SERONT DE MOINS BONNE QUALITE
; (LE CIEL DES IMAGES REDUITES NE SERA PAS AUSSI UNIFORME, SURTOUT EN BANDE K).
;
;##############################################################################
doflat:
for iteration = 1,4 do begin
	print, 'Reduit le set de flats #',iteration

	if iteration eq 1 then begin
		;Flats en K, 6 secondes
		nomFLATallume = faitnoms(path_imagesbrutes+'030327_', '0128-0134','.fits')
		nomFLATeteint = faitnoms(path_imagesbrutes+'030327_', '0135-0141','.fits')
		nomFLATfinal = filtreOS(path_imagescalibration+'flat_K_030327.fits')
		simon_flat, nomFLATallume, nomFLATeteint, nomFLATfinal
	endif
	if iteration eq 2 then begin
		;Flats en FeII, 60 secondes
		nomFLATallume = faitnoms(path_imagesbrutes+'030327_', '0142-0146','.fits')
		nomFLATeteint = faitnoms(path_imagesbrutes+'030327_', '0147-0151','.fits')
		nomFLATfinal = filtreOS(path_imagescalibration+'flat_FeII_030327.fits')
		simon_flat, nomFLATallume, nomFLATeteint, nomFLATfinal
	endif
	if iteration eq 3 then begin
		;Flats en H, 3 secondes
		nomFLATallume = faitnoms(path_imagesbrutes+'030327_', '0152-0157','.fits')
		nomFLATeteint = faitnoms(path_imagesbrutes+'030327_', '0158-0163','.fits')
		nomFLATfinal = filtreOS(path_imagescalibration+'flat_H_030327.fits')
		simon_flat, nomFLATallume, nomFLATeteint, nomFLATfinal
	endif
	;Voici un exemple si vous n'avez pas pris de flat avec lampe eteinte mais
	;que vous avez simplement un dark reduit (reduit lors de l'etape 3).
	if iteration eq 4 then begin
		;Flats en I, 25 secondes mais pas de flats avec lampe eteinte, seulement
		;un dark reduit de 25 secondes.
		nomFLATbrut = faitnoms(path_imagesbrutes+'030327_', '0168-0176','.fits')
		nomFLATfinal = filtreOS(path_imagescalibration+'flat_I_030327.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_25sec.fits')
		simon_flat, nomFLATbrut, nomDARK, nomFLATfinal
	endif

endfor



;##############################################################################
; ETAPE 5 - REDUCTION DES IMAGES SCIENTIFIQUES
;
; UNE FOIS QUE TOUTES LES IMAGES DE CALIBRATIONS SONT REDUITES, ON PEUT MAIN-
; TENANT REDUIRE LES IMAGES. IL EXISTE DIFFERENTS CHEMINS POUR Y PARVENIR QUI
; DEPENDENT PRINCIPALEMENT DU TYPE D'OBSERVATION EFFECTUEE. DEUX TYPES EXISTENT
; 1-LA CIBLE EST UN OBJET COMPACT OU UN CHAMP D'ETOILES PARSEMEES. DANS CE CAS,
; QUI EST LE PLUS SIMPLE, ON PEUT UTILISER LES IMAGES MEMES DE LA CIBLE POUR
; RECONSTRUIRE LE CIEL ET LE SOUSTRAIRE DE CHAQUE IMAGE. CONDITION FONDAMENTALE
; POUR QUE CA MARCHE: ON AURA FAIT DU DITHERING D'AU MOINS LA TAILLE DU PLUS
; GROS OBJET ENTRE CHAQUE POSE LORS DES OBSERVATIONS.
; 2-LA CIBLE EST UN OBJET DIFFUS ET ETENDU QUI OCCUPE LE QUART OU PLUS DU CHAMP.
; DANS CE CAS, IL FAUT EFFECTUER DES OBSERVATIONS DU CIEL EN PARALLELE (50% DU
; TEMPS) EN ALTERNANCE ENTRE L'OBJET ET UN CIEL PAS TROP LOIN.
;
; A PROPOS DU CIEL EN INFRAROUGE:
;
; LE CIEL EN INFRAROUGE EST TRES BRILLANT PAR RAPPORT AU CIEL DANS LE DOMAINE
; VISIBLE. C'EST CE QUI DOMINE LE SIGNAL DANS UNE IMAGE BRUTE ET CE QUI LIMITE
; VOS TEMPS DE POSE. MALHEUREUSEMENT, IL EST AUSSI TRES VARIABLE. SUR UNE ECHEL-
; LE DE TEMPS CARACTERISTIQUE DE 10-15 MINUTES, IL VARIE DE 10% EN AMPLITUDE
; (A CAUSE DES RAIES OH- ATMOSPHERIQUES PRINCIPALEMENT MAIS AUSSI A CAUSE DE
; L'EMISSION THERMIQUE DU CIEL ET DU TELESCOPE, REMARQUABLE EN BANDE K).
; PRESQUE TOUJOURS, LES PROBLEMES DE REDUCTION EN IR PROVIENNENT DE LA MAUVAISE
; SOUSTRACTION DU CIEL (DIAGNOSTIC: LE FOND DU CIEL N'EST PAS PLAT ET UNIFORME).
;
; CE SCRIPT TRAITE LES CAS NON-PROBLEMATIQUES OU LES OBSERVATIONS ONT ETE FAITES
; DANS LA REGLE DE L'ART, QUE LES CONDITIONS METEOS ETAIENT BONNES (PAS DE NUAGE
; OU DE BROUILLARD QUI AJOUTENT UN GRADIENT DANS LE FOND DU CIEL), OU SI ELLES
; ETAIENT VARIABLES, QUE L'OBSERVATEUR A PRIS DES IMAGES DU CIEL DE FACON REGU-
; LIERE ENTRE CELLES DE L'OBJET.
;
; LA BONNE FACON DE PRENDRE DES IMAGES:
;
; CAS D'UN OBJET PONCTUEL OU COMPACTE DANS UN CHAMP NON "CROWDE":
; 	A - FAIRE DU DITHERING ENTRE CHAQUE POSE. LA TAILLE DU DITHERING EST DE
;           L'ORDRE D'UNE DIZAINE DE SECONDES D'ARC OU DE LA TAILLE DE L'OBJET
;           LE PLUS ETENDU DANS VOTRE CHAMP.
;
; CAS D'UN OBJET ETENDU:
;	A - PRENDRE 3-4 POSES EN FAISANT DU DITHERING SUR L'OBJET
;	B - VOUS DEPLACER DE ~10 MINUTES D'ARC
;	C - PRENDRE 3-4 POSES EN FAISANT DU DITHERING SUR LE CIEL
;	D - REVENIR SUR L'OBJET
;	E - RECOMMENCER LA BOUCLES AUTANT DE FOIS QUE NECESSAIRE POUR ATTEINDRE
;	    LE SIGNAL SUR BRUIT REQUIS. NE REVENEZ PAS SUR LA MEME POSITION
;	    DE CIEL A CHAQUE FOIS, ALLEZ AU SUD, A L'EST, ETC.
;
; DEUX BONNES REFERENCES POUR LA REDUCTION EN INFRAROUGE:
;	"Strategies for Imaging Faint Extended Sources in the Near-Infrared"
;	Vaduvescu & McCall, 2004, PASP 116, 640
;	"Non-thermal emission in the atmosphere above Mauna Kea"
;	Ramsay, Mountain & Geballe, 1992, MNRAS 259, 751
;
;##############################################################################
doreduc:
for iteration = 6,6 do begin
	print, 'Reduit les images de la serie #',iteration

	;Exemple d'un objet ponctuel dans de bonnes conditions. Fabrique le ciel
	;a partir des images de l'objet elles-memes.
	if iteration eq 1 then begin
		;i - Defini quelles-sont ces images et les images de calibration
		nomOBJbrut = faitnoms(path_imagesbrutes+'030327_', '0056-0067','.fits')
		nomOBJredu = faitnoms(path_imagesreduites+'red030327_', '0056-0067', '.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_15sec.fits')
		nomFLAT = filtreOS(path_imagescalibration+'flat_I_030327.fits')
		nomBADPIX = filtreOS(path_imagescalibration+'badpix030212.fits')
		;ii - Appelle la procedure simon_objet pour d'abord faire la reduction
		;     des images, mais en specifiant de garder le ciel.
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		;iii - Appelle la procedure simon_ciel qui fabrique une image du ciel
		;      a partir des images reduites dont le ciel a ete conserve. Pour
		;      faire, il masque les etoiles avant de medianer les images.
		nomCIEL = filtreOS(path_imagescalibration+'ciel_M13I.fits')
		simon_ciel, nomOBJredutmp, nomCIEL
		;iv - Reduit pour vrai les images, cette fois en soustrayant le ciel.
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, nomCIEL=nomCIEL
	endif
	;Exemple d'une reduction entierement automatique ou le programme detecte le
	;filtre, les temps de pose, et les images de calibration a utiliser. Ici, le
	;programme va utiliser des flats, des darks de reference archivees sur cyga.
	;Pour que le programme utilise vos flats ou vos darks, voir l'exemple 3.
	if iteration eq 2 then begin
		path_imagesbrutes = '/home/tarqetoq/'
		;path_imagesreduites = '/home/loic/these/fig_standards/'
		;nomOBJbrut = faitnoms(path_imagesbrutes+'030616_','0051-0171,0174-0218','.fits.gz')
		nomOBJbrut = faitnoms(path_imagesbrutes+'030930_','0001-0255','.fits.gz')
		simon_automatique, nomOBJbrut
	endif
	if iteration eq 3 then begin
		path_imagesbrutes = '/home/tarqetoq/'
		;path_imagesreduites = '/home/loic/these/fig_standards/'
		;nomOBJbrut = faitnoms(path_imagesbrutes+'030616_','0051-0171,0174-0218','.fits.gz')
		nomOBJbrut = faitnoms(path_imagesbrutes+'030616_','0051-0171,0174-0305','.fits.gz')
		simon_automatique, nomOBJbrut
	endif
	if iteration eq 4 then begin
		path_imagesbrutes = '/home/tarqetoq/'
		path_imagesreduites = '/home/tarqetoq/test/'
		nomBADPIX = '/home/tarqetoq/test/badpix030212.fits'
		nomFLAT = '/home/tarqetoq/test/flat_J_030613.fits'
		nomCIEL = '/home/tarqetoq/test/ciel_030616_J.fits'
		;Reduit image 1 de 30sec
		nomOBJbrut = faitnoms(path_imagesbrutes+'030616_', '0001','.fits.gz')
		nomOBJredu = faitnoms(path_imagesreduites+'red030616_', '0001','.fits')
		nomDARK = '/home/tarqetoq/test/dark_030616_28.99sec.fits'
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, nomCIEL=nomCIEL
		;Reduit image 2 de 25sec
		nomOBJbrut = faitnoms(path_imagesbrutes+'030616_', '0002','.fits.gz')
		nomOBJredu = faitnoms(path_imagesreduites+'red030616_', '0002','.fits')
		nomDARK = '/home/tarqetoq/test/dark_030616_23.72sec.fits'
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, nomCIEL=nomCIEL
		;Reduit les autres de 20sec
		nomOBJbrut = faitnoms(path_imagesbrutes+'030616_', '0003-0014','.fits.gz')
		nomOBJredu = faitnoms(path_imagesreduites+'red030616_', '0003-0014','.fits')
		nomDARK = '/home/tarqetoq/test/dark_030616_23.72sec.fits'
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, nomCIEL=nomCIEL,$
			/ciel_plus_crepuscule
			
	endif
	if iteration eq 5 then begin
		path_imagesbrutes = '/home/tarqetoq/'
		path_imagesreduites = '/home/tarqetoq/test/'
		nomBADPIX = '/home/tarqetoq/test/badpix030212.fits'
		nomFLAT = '/home/tarqetoq/test/flat_J_030613.fits'
		nomDARK = '/home/tarqetoq/test/dark_030616_28.99sec.fits'
		nomOBJbrut = faitnoms(path_imagesbrutes+'030216_', '0127,0131,0133','.fits.gz')
		nomOBJredutmp = faitnoms(path_imagesreduites+'tmp030216_', '0127,0131,0133', '.fits')
		simon_objet, nomOBJbrut, nomOBJredutmp, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		nomCIEL = filtreOS(path_imagesreduites+'ciel_M92_J.fits')
		simon_ciel, nomOBJredutmp, nomCIEL,/iteratif
		nomOBJbrut = faitnoms(path_imagesbrutes+'030216_', '0127-0135','.fits.gz')
		nomOBJredu = faitnoms(path_imagesreduites+'red030216_', '0127-0135', '.fits')
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, nomCIEL=nomCIEL
	endif
	;Exemple d'une reduction d'un objet etendu qui a necessite l'observation du ciel a part.
	;Ca suppose que les observations ont ete menees intelligemment, ciel, objet, ciel, objet, etc.
	;ciel(0001-0003), objet(0004-0006), ciel(0007-0009), objet(0010-0012), ciel(0013-0015), etc.
	if iteration eq 6 then begin
		path_imagesbrutes = '/home/tarqetoq/'
		path_imagesreduites = '/home/tarqetoq/test/'
		nomBADPIX = '/home/tarqetoq/test/badpix040908.fits'
		nomFLAT = '/home/tarqetoq/test/flat_J_040908.fits'

		;fabrique le premier ciel (applicable pour le 1er set d'images sur l'objet)
		nomOBJbrut_ciel = faitnoms(path_imagesbrutes+'040908_', '0001-0003,0007-0009','.fits')
		nomOBJredu_ciel = faitnoms(path_imagesreduites+'tmp040908_', '0001-0003,0007-0009', '.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_objet, nomOBJbrut_ciel, nomOBJredu_ciel, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		nomCIEL = filtreOS(path_imagesreduites+'ciel_040908_1.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_ciel, nomOBJredu_ciel, nomCIEL
		;Soustrait le ciel du 1er set d'images sur l'objet
		nomOBJbrut = faitnoms(path_imagesbrutes+'040908_', '0004-0006','.fits')
		nomOBJredu = faitnoms(path_imagesreduites+'red040908_', '0004-0006', '.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /selon_temps

		;fabrique le 2e ciel (applicable pour le 2e set d'images sur l'objet)
		nomOBJbrut_ciel = faitnoms(path_imagesbrutes+'040908_', '0007-0009,0013-0015','.fits')
		nomOBJredu_ciel = faitnoms(path_imagesreduites+'tmp040908_', '0007-0009,0013-0015', '.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_objet, nomOBJbrut_ciel, nomOBJredu_ciel, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		nomCIEL = filtreOS(path_imagesreduites+'ciel_040908_2.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_ciel, nomOBJredu_ciel, nomCIEL
		;Soustrait le ciel du 2e set d'images sur l'objet
		nomOBJbrut = faitnoms(path_imagesbrutes+'040908_', '0010-0012','.fits')
		nomOBJredu = faitnoms(path_imagesreduites+'red040908_', '0010-0012', '.fits')
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /selon_temps

		;fabrique le 3e ciel (applicable pour le 3e set d'images sur l'objet)
		nomOBJbrut_ciel = faitnoms(path_imagesbrutes+'040908_', '0013-0015,0019-0021','.fits')
		nomOBJredu_ciel = faitnoms(path_imagesreduites+'tmp040908_', '0013-0015,0019-0021', '.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_objet, nomOBJbrut_ciel, nomOBJredu_ciel, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		nomCIEL = filtreOS(path_imagesreduites+'ciel_040908_3.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_ciel, nomOBJredu_ciel, nomCIEL
		;Soustrait le ciel du 3e set d'images sur l'objet
		nomOBJbrut = faitnoms(path_imagesbrutes+'040908_', '0016-0018','.fits')
		nomOBJredu = faitnoms(path_imagesreduites+'red040908_', '0016-0018', '.fits')
		nomDARK = '/home/tarqetoq/test/dark_040908_30sec.fits'
		simon_objet, nomOBJbrut, nomOBJredu, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /selon_temps
		;Ainsi de suite pour tous les sets... puis pour la combinaison des images, il faut utiliser
		;les images red*.fits comme d'habitude (voir les exemples de combinaison). 
	endif

endfor

end




pro fabrication_image_de_franges

for reduit_frange = 4,4 do begin
	print, 'Fabrique l''image des franges de la serie #',reduit_frange
	;Franges en I avec les images de M13 - marche pas, il reste plein d'etoiles.
	if reduit_frange eq 1 then begin
		nomOBJbrut = faitnoms(path_imagesbrutes+'030327_', '0057,0060,0063,0066','.fits')
		nomOBJff = faitnoms(path_imagesreduites+'tmp030327_', '0057,0060,0063,0066','.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_15sec.fits')
		nomFLAT = filtreOS(path_imagescalibration+'flat_I_030327.fits')
		nomBADPIX = filtreOS(path_imagescalibration+'badpix030212.fits')
		nomFRANGE = filtreOS(path_imagescalibration+'frange_I_M13.fits')
		simon_champ_stellaire, nomOBJbrut, nomOBJff, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		simon_frange, nomOBJff, nomFRANGE
	endif
	;Franges en I avec les images de M57.
	if reduit_frange eq 2 then begin
		nomOBJbrut = faitnoms(path_imagesbrutes+'030327_', '0093','.fits')
		nomOBJff = faitnoms(path_imagesreduites+'tmp030327_', '0093','.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_15sec.fits')
		nomFLAT = filtreOS(path_imagescalibration+'flat_I_030327.fits')
		nomBADPIX = filtreOS(path_imagescalibration+'badpix030212.fits')
		nomFRANGE = filtreOS(path_imagescalibration+'frange_I_M57.fits')
		simon_champ_stellaire, nomOBJbrut, nomOBJff, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		nomOBJbrut = faitnoms(path_imagesbrutes+'030327_', '0094-0097','.fits')
		nomOBJff = faitnoms(path_imagesreduites+'tmp030327_', '0094-0097','.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_30sec.fits')
		simon_champ_stellaire, nomOBJbrut, nomOBJff, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		nomOBJbrut = faitnoms(path_imagesbrutes+'030327_', '0098-0099','.fits')
		nomOBJff = faitnoms(path_imagesreduites+'tmp030327_', '0098-0099','.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_120sec.fits')
		simon_champ_stellaire, nomOBJbrut, nomOBJff, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		nomOBJff = faitnoms(path_imagesreduites+'tmp030327_', '0094-0099','.fits')
		simon_frange, nomOBJff, nomFRANGE
	endif
	;Franges en K avec les images de Pluton.
	if reduit_frange eq 3 then begin
		nomOBJbrut = faitnoms(path_imagesbrutes+'030327_', '0126-0134','.fits')
		nomOBJff = faitnoms(path_imagesreduites+'tmp030327_', '0126-0134','.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_6sec.fits')
		nomFLAT = filtreOS(path_imagescalibration+'flat_K_030327.fits')
		nomBADPIX = filtreOS(path_imagescalibration+'badpix030212.fits')
		nomFRANGE = filtreOS(path_imagescalibration+'frange_K_Pluton.fits')
		simon_champ_stellaire, nomOBJbrut, nomOBJff, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		simon_frange, nomOBJff, nomFRANGE
	endif
	;Franges en K avec les images de M13 deja traitees.
	if reduit_frange eq 4 then begin
		nomOBJbrut = faitnoms(path_imagesbrutes+'030327_', '0081-0092','.fits')
		nomOBJff = faitnoms(path_imagesreduites+'tmp030327_', '0081-0092','.fits')
		nomFRANGE = filtreOS(path_imagescalibration+'frange_K_M13.fits')
		nomDARK = filtreOS(path_imagescalibration+'dark_blank_30sec.fits')
		nomFLAT = filtreOS(path_imagescalibration+'flat_K_030327.fits')
		nomBADPIX = filtreOS(path_imagescalibration+'badpix030212.fits')
		simon_champ_stellaire, nomOBJbrut, nomOBJff, nomFLAT, nomDARK, nomBADPIX=nomBADPIX, /garde_ciel
		simon_frange, nomOBJff, nomFRANGE
	endif




endfor

end




pro combinaison_des_images
;common paths, path_imagesbrutes, path_imagesreduites, path_imagescalibration

for iteration = 8,8 do begin
	print, 'Combine les images du set #',iteration

	if iteration eq 1 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030930_','0058-0076','.fits')
		nomMOZZ = 'objet1_J.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	if iteration eq 2 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030930_','0077-0086,0088-0090,0092-0118','.fits')
		nomMOZZ = 'objet0077-0118_H.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	if iteration eq 3 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030930_','0119-0141','.fits')
		nomMOZZ = 'objet0119-0141_Ks.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	if iteration eq 4 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030930_','0142-0149,0151-0152','.fits')
		nomMOZZ = 'objet0142-0152_J.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	if iteration eq 5 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030930_','0153-0161','.fits')
		nomMOZZ = 'as08_0153-0161_H.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	if iteration eq 6 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030930_','0162-0177','.fits')
		nomMOZZ = 'as08_0162-0177_Ks.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	;if iteration eq 7 then begin
	;	nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030930_','0178-0183','.fits')
	;	nomMOZZ = 'as08_0178-0183_J.fits'
	;	refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
	;	imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
	;		listex=x, listey=y
	;endif
	if iteration eq 8 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030616_','0219-0232','.fits')
		nomMOZZ = 'm57_030616_J.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	if iteration eq 8 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030616_','0236-0243','.fits')
		nomMOZZ = 'm57_030616_I.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	if iteration eq 8 then begin
		nomOBJreduit = faitnoms('/home/tarqetoq/test/'+'red030616_','0246-0253','.fits')
		nomMOZZ = 'm57_030616_PaBeta.fits'
		refstar, nomOBJreduit, x,y, resolelement=5.0,/absolu
		imsum, nomOBJreduit, nomMOZZ, 'LISTE', /mediane, resolelement=5.0, $
			listex=x, listey=y
	endif
	


endfor

end
